@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*-
  * Copyright (c) 1980, 1993
@@ -38,12 +38,15 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)cmd1.c	2.22 (gritter) 3/30/03";
+static char sccsid[] = "@(#)cmd1.c	2.94 (gritter) 3/4/05";
 #endif
 #endif /* not lint */
 
 #include "rcv.h"
 #include "extern.h"
+#ifdef	HAVE_WCWIDTH
+#include <wchar.h>
+#endif
 
 /*
  * Mail -- a mail program
@@ -57,51 +60,133 @@ static char sccsid[] = "@(#)cmd1.c	2.22 (gritter) 3/30/03";
  */
 
 static int screen;
-static RETSIGTYPE brokpipe __P((int));
-static int	pipe1 __P((char *, int));
-static int	type1 __P((int *, int, int, int, char *, off_t *));
+static void onpipe(int signo);
+static int dispc(struct message *mp, const char *a);
+static int scroll1(char *arg, int onlynew);
+static void hprf(const char *fmt, int mesg, FILE *f, int threaded,
+		const char *attrlist);
+static int putindent(FILE *fp, struct message *mp, int maxwidth);
+static int type1(int *msgvec, int doign, int page, int pipe, int decode,
+		char *cmd, off_t *tstats);
+static int pipe1(char *str, int doign);
+void brokpipe(int signo);
 
 char *
-get_pager()
+get_pager(void)
 {
 	char *cp;
 
 	cp = value("PAGER");
 	if (cp == NULL || *cp == '\0')
-		cp = value("bsdcompat") ? PATH_MORE : PATH_PG;
+		cp = value("bsdcompat") ? "more" : "pg";
 	return cp;
 }
-int
-headers(v)
-	void *v;
+
+int 
+headers(void *v)
 {
 	int *msgvec = v;
-	int n, mesg, flag;
-	struct message *mp;
+	int g, k, n, mesg, flag = 0, lastg = 1;
+	struct message *mp, *mq, *lastmq = NULL;
 	int size;
+	enum mflag	fl = MNEW|MFLAGGED;
 
 	size = screensize();
-	n = msgvec[0];
-	if (n != 0)
-		screen = (n-1)/size;
+	n = msgvec[0];	/* n == {-2, -1, 0}: called from scroll() */
 	if (screen < 0)
 		screen = 0;
-	mp = &message[screen * size];
-	if (mp >= &message[msgcount])
-		mp = &message[msgcount - size];
-	if (mp < &message[0])
-		mp = &message[0];
-	flag = 0;
-	mesg = mp - &message[0];
-	if (dot != &message[n-1])
-		setdot(mp);
-	for (; mp < &message[msgcount]; mp++) {
-		mesg++;
-		if (mp->m_flag & MDELETED)
-			continue;
-		if (flag++ >= size)
-			break;
-		printhead(mesg, stdout);
+	k = screen * size;
+	if (k >= msgCount)
+		k = msgCount - size;
+	if (k < 0)
+		k = 0;
+	if (mb.mb_threaded == 0) {
+		g = 0;
+		mq = &message[0];
+		for (mp = &message[0]; mp < &message[msgCount]; mp++)
+			if ((mp->m_flag&(MDELETED|MHIDDEN|MKILL))==0) {
+				if (g % size == 0)
+					mq = mp;
+				if (mp->m_flag&fl) {
+					lastg = g;
+					lastmq = mq;
+				}
+				if (n>0 && mp==&message[n-1] ||
+						n==0 && g==k ||
+						n==-2 && g==k+size && lastmq ||
+						n<0 && g>=k && mp->m_flag&fl)
+					break;
+				g++;
+			}
+		if (lastmq && (n==-2 || n==-1 && mp==&message[msgCount])) {
+			g = lastg;
+			mq = lastmq;
+		}
+		screen = g / size;
+		mp = mq;
+		mesg = mp - &message[0];
+		if (dot != &message[n-1]) {
+			for (mq = mp; mq < &message[msgCount]; mq++)
+				if ((mq->m_flag&(MDELETED|MHIDDEN|MKILL))==0) {
+					setdot(mq);
+					break;
+				}
+		}
+		if (mb.mb_type == MB_IMAP)
+			imap_getheaders(mesg+1, mesg + size);
+		for (; mp < &message[msgCount]; mp++) {
+			mesg++;
+			if (mp->m_flag & (MDELETED|MHIDDEN|MKILL))
+				continue;
+			if (flag++ >= size)
+				break;
+			printhead(mesg, stdout, 0);
+		}
+	} else {	/* threaded */
+		g = 0;
+		mq = threadroot;
+		for (mp = threadroot; mp; mp = next_in_thread(mp))
+			if ((mp->m_flag&(MDELETED|MHIDDEN|MKILL))==0 &&
+					(mp->m_collapsed <= 0 ||
+					 mp == &message[n-1])) {
+				if (g % size == 0)
+					mq = mp;
+				if (mp->m_flag&fl) {
+					lastg = g;
+					lastmq = mq;
+				}
+				if (n>0 && mp==&message[n-1] ||
+						n==0 && g==k ||
+						n==-2 && g==k+size && lastmq ||
+						n<0 && g>=k && mp->m_flag&fl)
+					break;
+				g++;
+			}
+		if (lastmq && (n==-2 || n==-1 && mp==&message[msgCount])) {
+			g = lastg;
+			mq = lastmq;
+		}
+		screen = g / size;
+		mp = mq;
+		if (dot != &message[n-1]) {
+			for (mq = mp; mq; mq = next_in_thread(mq))
+				if ((mq->m_flag&(MDELETED|MHIDDEN|MKILL))==0 &&
+						mq->m_collapsed <= 0) {
+					setdot(mq);
+					break;
+				}
+		}
+		while (mp) {
+			if ((mp->m_flag & (MDELETED|MHIDDEN|MKILL)) == 0 &&
+					(mp->m_collapsed <= 0 ||
+					 mp == &message[n-1])) {
+				if (flag++ >= size)
+					break;
+				printhead(mp - &message[0] + 1, stdout,
+						mb.mb_threaded);
+			}
+			mp = next_in_thread(mp);
+		}
 	}
 	if (flag == 0) {
 		printf(catgets(catd, CATSET, 6, "No more mail.\n"));
@@ -114,14 +199,24 @@ headers(v)
  * Scroll to the next/previous screen
  */
 int
-scroll(v)
-	void *v;
+scroll(void *v)
+{
+	return scroll1(v, 0);
+}
+
+int
+Scroll(void *v)
+{
+	return scroll1(v, 1);
+}
+
+static int
+scroll1(char *arg, int onlynew)
 {
-	char *arg = v;
 	int size;
 	int cur[1];
 
-	cur[0] = 0;
+	cur[0] = onlynew ? -1 : 0;
 	size = screensize();
 	switch (*arg) {
 	case '1': case '2': case '3': case '4': case '5':
@@ -132,7 +227,7 @@ scroll(v)
 		screen++;
 		goto scroll_forward;
 	case '$':
-		screen = msgcount / size;
+		screen = msgCount / size;
 		goto scroll_forward;
 	case '+':
 		if (arg[1] == '\0')
@@ -140,8 +235,8 @@ scroll(v)
 		else
 			screen += atoi(arg + 1);
 scroll_forward:
-		if (screen * size > msgcount) {
-			screen = msgcount / size;
+		if (screen * size > msgCount) {
+			screen = msgCount / size;
 			printf(catgets(catd, CATSET, 7,
 					"On last screenful of messages\n"));
 		}
@@ -157,6 +252,8 @@ scroll_forward:
 			printf(catgets(catd, CATSET, 8,
 					"On first screenful of messages\n"));
 		}
+		if (cur[0] == -1)
+			cur[0] = -2;
 		break;
 
 	default:
@@ -170,8 +267,8 @@ scroll_forward:
 /*
  * Compute screen size.
  */
-int
-screensize()
+int 
+screensize(void)
 {
 	int s;
 	char *cp;
@@ -184,8 +281,8 @@ screensize()
 static sigjmp_buf	pipejmp;
 
 /*ARGSUSED*/
-static RETSIGTYPE
-onpipe(signo)
+static void 
+onpipe(int signo)
 {
 	siglongjmp(pipejmp, 1);
 }
@@ -194,9 +291,8 @@ onpipe(signo)
  * Print out the headlines for each message
  * in the passed message list.
  */
-int
-from(v)
-	void *v;
+int 
+from(void *v)
 {
 	int *msgvec = v;
 	int *ip, n;
@@ -220,7 +316,7 @@ from(v)
 		}
 	}
 	for (ip = msgvec; *ip != 0; ip++)
-		printhead(*ip, obuf);
+		printhead(*ip, obuf, mb.mb_threaded);
 	if (--ip >= msgvec)
 		setdot(&message[*ip - 1]);
 endpipe:
@@ -232,32 +328,61 @@ endpipe:
 	return(0);
 }
 
-/*
- * Print out the header of a specific message.
- * This is a slight improvement to the standard one.
- */
-void
-printhead(mesg, f)
-	int mesg;
-	FILE *f;
+static int 
+dispc(struct message *mp, const char *a)
 {
-	struct message *mp;
-	char *headline = NULL, lcount[64], ccount[64], *subjline, dispc, curind;
-	size_t headsize = 0;
-	int headlen = 0;
-	char *pbuf = NULL;
-	struct headline hl;
-	struct str in, out;
-	int subjlen, fromlen, isto = 0, isaddr = 0;
-	int bsdflags, bsdheadline;
-	char *name, *cp;
-	FILE *ibuf;
+	int	dispc = ' ';
+
+	/*
+	 * Bletch!
+	 */
+	if ((mp->m_flag & (MREAD|MNEW)) == MREAD)
+		dispc = a[3];
+	if ((mp->m_flag & (MREAD|MNEW)) == (MREAD|MNEW))
+		dispc = a[2];
+	if (mp->m_flag & MANSWERED)
+		dispc = a[8];
+	if (mp->m_flag & MDRAFTED)
+		dispc = a[9];
+	if ((mp->m_flag & (MREAD|MNEW)) == MNEW)
+		dispc = a[0];
+	if ((mp->m_flag & (MREAD|MNEW)) == 0)
+		dispc = a[1];
+	if (mp->m_flag & MJUNK)
+		dispc = a[13];
+	if (mp->m_flag & MSAVED)
+		dispc = a[4];
+	if (mp->m_flag & MPRESERVE)
+		dispc = a[5];
+	if (mp->m_flag & (MBOX|MBOXED))
+		dispc = a[6];
+	if (mp->m_flag & MFLAGGED)
+		dispc = a[7];
+	if (mp->m_flag & MKILL)
+		dispc = a[10];
+	if (mb.mb_threaded == 1 && mp->m_collapsed > 0)
+		dispc = a[12];
+	if (mb.mb_threaded == 1 && mp->m_collapsed < 0)
+		dispc = a[11];
+	return dispc;
+}
+
+static void
+hprf(const char *fmt, int mesg, FILE *f, int threaded, const char *attrlist)
+{
+	struct message	*mp = &message[mesg-1];
+	char	*headline = NULL, *subjline, *name, *cp, *pbuf = NULL;
+	struct headline	hl;
+	size_t	headsize = 0;
+	const char	*fp;
+	int	B, c, i, n, s;
+	int	headlen = 0;
+	struct str	in, out;
+	int	subjlen = scrnwidth, fromlen, isto = 0, isaddr = 0;
+	FILE	*ibuf;
 
-	mp = &message[mesg-1];
-	bsdflags = value("bsdcompat") != NULL || value("bsdflags") != NULL;
-	bsdheadline= value("bsdcompat") != NULL || value("bsdheadline") != NULL;
 	if ((mp->m_flag & MNOFROM) == 0) {
-		if ((ibuf = setinput(mp, NEED_HEADER)) == NULL)
+		if ((ibuf = setinput(&mb, mp, NEED_HEADER)) == NULL)
 			return;
 		if ((headlen = readline(ibuf, &headline, &headsize)) < 0)
 			return;
@@ -265,7 +390,6 @@ printhead(mesg, f)
 	if ((subjline = hfield("subject", mp)) == NULL)
 		subjline = hfield("subj", mp);
 	if (subjline == NULL) {
-		subjline = "";
 		out.s = NULL;
 		out.l = 0;
 	} else {
@@ -274,27 +398,6 @@ printhead(mesg, f)
 		mime_fromhdr(&in, &out, TD_ICONV | TD_ISPR);
 		subjline = out.s;
 	}
-	/*
-	 * Bletch!
-	 */
-	curind = dot == mp ? '>' : ' ';
-	dispc = ' ';
-	if (bsdflags == 0) {
-		if (mp->m_flag & (MREAD|MNEW))
-			dispc = 'R';
-		if ((mp->m_flag & (MREAD|MNEW)) == MREAD)
-			dispc = 'O';
-	}
-	if (mp->m_flag & MSAVED)
-		dispc = bsdflags ? '*' : 'S';
-	if (mp->m_flag & MPRESERVE)
-		dispc = bsdflags ? 'P' : 'H';
-	if ((mp->m_flag & (MREAD|MNEW)) == MNEW)
-		dispc = 'N';
-	if ((mp->m_flag & (MREAD|MNEW)) == 0)
-		dispc = 'U';
-	if (mp->m_flag & MBOX)
-		dispc = 'M';
 	if ((mp->m_flag & MNOFROM) == 0) {
 		pbuf = ac_alloc(headlen + 1);
 		parse(headline, headlen, &hl, pbuf);
@@ -305,20 +408,11 @@ printhead(mesg, f)
 	}
 	if (value("datefield") && (cp = hfield("date", mp)) != NULL)
 		hl.l_date = fakedate(rfctime(cp));
-	if (mp->m_xlines > 0)
-		snprintf(lcount, sizeof lcount, "%3d", mp->m_xlines);
-	else
-		strcpy(lcount, "   ");
-	snprintf(ccount, sizeof ccount, "%-5u", (unsigned)mp->m_xsize);
-	subjlen = scrnwidth - (bsdheadline ? 49 : 45) - strlen(ccount) -
-		strlen(ccount);
-	if (subjlen > out.l)
-		subjlen = out.l;
 	if (Iflag) {
 		if ((name = hfield("newsgroups", mp)) == NULL)
 			if ((name = hfield("article-id", mp)) == NULL)
 				name = "<>";
-		name = makeprint0(name);
+		name = prstr(name);
 	} else if (value("show-rcpt") == NULL) {
 		name = name1(mp, 0);
 		isaddr = 1;
@@ -337,34 +431,169 @@ printhead(mesg, f)
 		name = "";
 		isaddr = 0;
 	}
-	if (bsdheadline)
-		fromlen = isto ? 17 : 20;
-	else
-		fromlen = isto ? 15 : 18;
 	if (isaddr) {
 		if (value("showname"))
 			name = realname(name);
 		else {
-			name = makeprint0(skin(name));
+			name = prstr(skin(name));
 		}
 	}
-	if (subjline == NULL || subjlen < 0) {         /* pretty pathetic */
-		fprintf(f, bsdheadline ?  catgets(catd, CATSET, 206,
-				"%c%c%3d %s%-*.*s  %16.16s %s/%s\n") :
-			catgets(catd, CATSET, 11,
-				"%c%c%3d %s%-*.*s  %16.16s %s/%s\n"),
-			curind, dispc, mesg, isto ? "To " : "",
-			fromlen, fromlen, name, hl.l_date, lcount, ccount);
-	} else {
-		makeprint(subjline, subjlen);
-		fprintf(f, bsdheadline ? catgets(catd, CATSET, 207,
-				"%c%c%3d %s%-*.*s  %16.16s %s/%s \"%.*s\"\n") :
-			catgets(catd, CATSET, 12,
-				"%c%c%3d %s%-*.*s  %16.16s %s/%s %.*s\n"),
-			curind, dispc, mesg, isto ? "To " : "",
-			fromlen, fromlen, name, hl.l_date, lcount, ccount,
-			subjlen, subjline);
+	for (fp = fmt; *fp; fp++) {
+		if (*fp == '%') {
+			if (*++fp == '-') {
+				fp++;
+			} else if (*fp == '+')
+				fp++;
+			while (digitchar(*fp&0377))
+				fp++;
+			if (*fp == '\0')
+				break;
+		} else {
+#if defined (HAVE_MBTOWC) && defined (HAVE_WCWIDTH)
+			if (mb_cur_max > 1) {
+				wchar_t	wc;
+				if ((s = mbtowc(&wc, fp, mb_cur_max)) < 0)
+					n = s = 1;
+				else {
+					if ((n = wcwidth(wc)) < 0)
+						n = 1;
+				}
+			} else
+#endif  /* HAVE_MBTOWC && HAVE_WCWIDTH */
+			{
+				n = s = 1;
+			}
+			subjlen -= n;
+			while (--s > 0)
+				fp++;
+		}
 	}
+	for (fp = fmt; *fp; fp++) {
+		if (*fp == '%') {
+			B = 0;
+			n = 0;
+			s = 1;
+			if (*++fp == '-') {
+				s = -1;
+				fp++;
+			} else if (*fp == '+')
+				fp++;
+			if (digitchar(*fp&0377)) {
+				do
+					n = 10*n + *fp - '0';
+				while (fp++, digitchar(*fp&0377));
+			}
+			if (*fp == '\0')
+				break;
+			n *= s;
+			switch (*fp) {
+			case '%':
+				putc('%', f);
+				subjlen--;
+				break;
+			case '>':
+			case '<':
+				c = dot == mp ? *fp&0377 : ' ';
+				putc(c, f);
+				subjlen--;
+				break;
+			case 'a':
+				c = dispc(mp, attrlist);
+				putc(c, f);
+				subjlen--;
+				break;
+			case 'm':
+				if (n == 0) {
+					n = 3;
+					if (threaded)
+						for (i=msgCount; i>999; i/=10)
+							n++;
+				}
+				subjlen -= fprintf(f, "%*d", n, mesg);
+				break;
+			case 'f':
+				if (n <= 0)
+					n = 18;
+				fromlen = n;
+				if (isto)
+					fromlen -= 3;
+				fprintf(f, "%s%s", isto ? "To " : "",
+						colalign(name, fromlen, 1));
+				subjlen -= n;
+				break;
+			case 'd':
+				if (n <= 0)
+					n = 16;
+				subjlen -= fprintf(f, "%*.*s", n, n, hl.l_date);
+				break;
+			case 'l':
+				if (n == 0)
+					n = 4;
+				if (mp->m_xlines)
+					subjlen -= fprintf(f, "%*ld", n,
+							mp->m_xlines);
+				else {
+					subjlen -= n;
+					while (n--)
+						putc(' ', f);
+				}
+				break;
+			case 'o':
+				if (n == 0)
+					n = -5;
+				subjlen -= fprintf(f, "%*lu", n,
+						(long)mp->m_xsize);
+				break;
+			case 'i':
+				if (threaded)
+					subjlen -= putindent(f, mp,
+							scrnwidth - 60);
+				break;
+			case 'S':
+				B = 1;
+				/*FALLTHRU*/
+			case 's':
+				n = n>0 ? n : subjlen - 2;
+				if (B)
+					n -= 2;
+				if (subjline != NULL && n >= 0) {
+					/* pretty pathetic */
+					fprintf(f, B ? "\"%s\"" : "%s",
+						colalign(subjline, n, 0));
+				}
+				break;
+			case 'U':
+				if (n == 0)
+					n = 9;
+				subjlen -= fprintf(f, "%*lu", n, mp->m_uid);
+				break;
+			case 'e':
+				if (n == 0)
+					n = 2;
+				subjlen -= fprintf(f, "%*u", n, threaded == 1 ?
+						mp->m_level : 0);
+				break;
+			case 't':
+				if (n == 0) {
+					n = 3;
+					if (threaded)
+						for (i=msgCount; i>999; i/=10)
+							n++;
+				}
+				fprintf(f, "%*ld", n, threaded ?
+						mp->m_threadpos : mesg);
+				subjlen -= n;
+				break;
+			case 'c':
+				if (n == 0)
+					n = 6;
+				subjlen -= fprintf(f, "%*g", n, mp->m_score);
+				break;
+			}
+		} else
+			putc(*fp&0377, f);
+	}
+	putc('\n', f);
 	if (out.s)
 		free(out.s);
 	if (headline)
@@ -373,13 +602,101 @@ printhead(mesg, f)
 		ac_free(pbuf);
 }
 
+/*
+ * Print out the indenting in threaded display.
+ */
+static int
+putindent(FILE *fp, struct message *mp, int maxwidth)
+{
+	struct message	*mq;
+	int	indent, i;
+	int	*us;
+	char	*cs;
+	int	important = MNEW|MFLAGGED;
+
+	if (mp->m_level == 0)
+		return 0;
+	cs = ac_alloc(mp->m_level);
+	us = ac_alloc(mp->m_level * sizeof *us);
+	i = mp->m_level - 1;
+	if (mp->m_younger && mp->m_younger->m_level == i + 1) {
+		if (mp->m_parent && mp->m_parent->m_flag & important)
+			us[i] = mp->m_flag & important ? 0x2523 : 0x2520;
+		else
+			us[i] = mp->m_flag & important ? 0x251D : 0x251C;
+		cs[i] = '+';
+	} else {
+		if (mp->m_parent && mp->m_parent->m_flag & important)
+			us[i] = mp->m_flag & important ? 0x2517 : 0x2516;
+		else
+			us[i] = mp->m_flag & important ? 0x2515 : 0x2514;
+		cs[i] = '\\';
+	}
+	mq = mp->m_parent;
+	for (i = mp->m_level - 2; i >= 0; i--) {
+		if (mq) {
+			if (i > mq->m_level - 1) {
+				us[i] = cs[i] = ' ';
+				continue;
+			}
+			if (mq->m_younger) {
+				if (mq->m_parent &&
+						mq->m_parent->m_flag&important)
+					us[i] = 0x2503;
+				else
+					us[i] = 0x2502;
+				cs[i] = '|';
+			} else
+				us[i] = cs[i] = ' ';
+			mq = mq->m_parent;
+		} else
+			us[i] = cs[i] = ' ';
+	}
+	for (indent = 0; indent < mp->m_level && indent < maxwidth; indent++) {
+		if (indent < maxwidth - 1)
+			putuc(us[indent], cs[indent] & 0377, fp);
+		else
+			putuc(0x21B8, '^', fp);
+	}
+	ac_free(us);
+	ac_free(cs);
+	return indent;
+}
+
+/*
+ * Print out the header of a specific message.
+ * This is a slight improvement to the standard one.
+ */
+void
+printhead(int mesg, FILE *f, int threaded)
+{
+	int bsdflags, bsdheadline, sz;
+	char	*fmt, attrlist[30], *cp;
+
+	bsdflags = value("bsdcompat") != NULL || value("bsdflags") != NULL ||
+		getenv("SYSV3") != NULL;
+	strcpy(attrlist, bsdflags ? "NU  *HMFATK+-J" : "NUROSPMFATK+-J");
+	if ((cp = value("attrlist")) != NULL) {
+		sz = strlen(cp);
+		if (sz > sizeof attrlist - 1)
+			sz = sizeof attrlist - 1;
+		memcpy(attrlist, cp, sz);
+	}
+	bsdheadline = value("bsdcompat") != NULL ||
+		value("bsdheadline") != NULL;
+	if ((fmt = value("headline")) == NULL)
+		fmt = bsdheadline ?
+			"%>%a%m %20f  %16d %3l/%-5o %i%S" :
+			"%>%a%m %18f %16d %4l/%-5o %i%s";
+	hprf(fmt, mesg, f, threaded, attrlist);
+}
+
 /*
  * Print out the value of dot.
  */
 /*ARGSUSED*/
-int
-pdot(v)
-	void *v;
+int 
+pdot(void *v)
 {
 	printf(catgets(catd, CATSET, 13, "%d\n"),
 			(int)(dot - &message[0] + 1));
@@ -390,9 +707,8 @@ pdot(v)
  * Print out all the possible commands.
  */
 /*ARGSUSED*/
-int
-pcmdlist(v)
-	void *v;
+int 
+pcmdlist(void *v)
 {
 	extern const struct cmd cmdtab[];
 	const struct cmd *cp;
@@ -419,10 +735,8 @@ pcmdlist(v)
 static sigjmp_buf	pipestop;
 
 static int
-type1(msgvec, doign, page, pipe, cmd, tstats)
-int *msgvec;
-char *cmd;
-off_t *tstats;
+type1(int *msgvec, int doign, int page, int pipe, int decode,
+		char *cmd, off_t *tstats)
 {
 	int *ip;
 	struct message *mp;
@@ -446,9 +760,9 @@ off_t *tstats;
 	if (pipe) {
 		cp = value("SHELL");
 		if (cp == NULL)
-			cp = PATH_CSHELL;
+			cp = SHELL;
 		obuf = Popen(cmd, "w", cp, 1);
-		if (obuf == (FILE*)NULL) {
+		if (obuf == NULL) {
 			perror(cmd);
 			obuf = stdout;
 		} else {
@@ -458,7 +772,7 @@ off_t *tstats;
 	    (page || (cp = value("crt")) != NULL)) {
 		nlines = 0;
 		if (!page) {
-			for (ip = msgvec; *ip && ip-msgvec < msgcount; ip++) {
+			for (ip = msgvec; *ip && ip-msgvec < msgCount; ip++) {
 				if ((message[*ip-1].m_have & HAVE_BODY) == 0) {
 					if ((get_body(&message[*ip - 1])) !=
 							OKAY)
@@ -470,26 +784,28 @@ off_t *tstats;
 		if (page || nlines > (*cp ? atoi(cp) : realscreenheight)) {
 			cp = get_pager();
 			obuf = Popen(cp, "w", NULL, 1);
-			if (obuf == (FILE*)NULL) {
+			if (obuf == NULL) {
 				perror(cp);
 				obuf = stdout;
 			} else
 				safe_signal(SIGPIPE, brokpipe);
 		}
 	}
-	for (ip = msgvec; *ip && ip - msgvec < msgcount; ip++) {
+	for (ip = msgvec; *ip && ip - msgvec < msgCount; ip++) {
 		mp = &message[*ip - 1];
 		touch(mp);
 		setdot(mp);
+		uncollapse1(mp, 1);
 		if (value("quiet") == NULL)
 			fprintf(obuf, catgets(catd, CATSET, 17,
 				"Message %2d:\n"), *ip);
-		(void) send_message(mp, obuf, doign ? ignore : 0, NULL,
-				    pipe && value("piperaw") ?
-				    	CONV_NONE : CONV_TODISP,
-				    mstats);
+		send(mp, obuf, doign ? ignore : 0, NULL,
+			pipe && value("piperaw") ? SEND_MBOX :
+				decode ? SEND_SHOW :
+				doign ? SEND_TODISP : SEND_TODISP_ALL,
+			mstats);
 		if (pipe && value("page")) {
-			sputc('\f', obuf);
+			putc('\f', obuf);
 		}
 		if (tstats) {
 			tstats[0] += mstats[0];
@@ -512,9 +828,7 @@ close_pipe:
  * Get the last, possibly quoted part of linebuf.
  */
 char *
-laststring(linebuf, flag, strip)
-char *linebuf;
-int *flag;
+laststring(char *linebuf, int *flag, int strip)
 {
 	char *cp, *p;
 	char quoted;
@@ -579,16 +893,15 @@ int *flag;
 /*
  * Pipe the messages requested.
  */
-static int
-pipe1(str, doign)
-char *str;
+static int 
+pipe1(char *str, int doign)
 {
 	char *cmd;
 	int f, *msgvec, ret;
 	off_t stats[2];
 
 	/*LINTED*/
-	msgvec = (int *)salloc((msgcount + 2) * sizeof *msgvec);
+	msgvec = (int *)salloc((msgCount + 2) * sizeof *msgvec);
 	if ((cmd = laststring(str, &f, 1)) == NULL) {
 		cmd = value("cmd");
 		if (cmd == NULL || *cmd == '\0') {
@@ -600,15 +913,23 @@ char *str;
 	if (!f) {
 		*msgvec = first(0, MMNORM);
 		if (*msgvec == 0) {
+			if (inhook)
+				return 0;
 			puts(catgets(catd, CATSET, 18, "No messages to pipe."));
 			return 1;
 		}
 		msgvec[1] = 0;
 	} else if (getmsglist(str, msgvec, 0) < 0)
 		return 1;
+	if (*msgvec == 0) {
+		if (inhook)
+			return 0;
+		printf("No applicable messages.\n");
+		return 1;
+	}
 	printf(catgets(catd, CATSET, 268, "Pipe to: \"%s\"\n"), cmd);
 	stats[0] = stats[1] = 0;
-	if ((ret = type1(msgvec, doign, 0, 1, cmd, stats)) == 0) {
+	if ((ret = type1(msgvec, doign, 0, 1, 0, cmd, stats)) == 0) {
 		printf("\"%s\" ", cmd);
 		if (stats[0] >= 0)
 			printf("%lu", (long)stats[0]);
@@ -622,56 +943,62 @@ char *str;
 /*
  * Paginate messages, honor ignored fields.
  */
-int
-more(v)
-	void *v;
+int 
+more(void *v)
 {
 	int *msgvec = v;
-	return (type1(msgvec, 1, 1, 0, NULL, NULL));
+	return (type1(msgvec, 1, 1, 0, 0, NULL, NULL));
 }
 
 /*
  * Paginate messages, even printing ignored fields.
  */
-int
-More(v)
-	void *v;
+int 
+More(void *v)
 {
 	int *msgvec = v;
 
-	return (type1(msgvec, 0, 1, 0, NULL, NULL));
+	return (type1(msgvec, 0, 1, 0, 0, NULL, NULL));
 }
 
 /*
  * Type out messages, honor ignored fields.
  */
-int
-type(v)
-	void *v;
+int 
+type(void *v)
 {
 	int *msgvec = v;
 
-	return(type1(msgvec, 1, 0, 0, NULL, NULL));
+	return(type1(msgvec, 1, 0, 0, 0, NULL, NULL));
 }
 
 /*
  * Type out messages, even printing ignored fields.
  */
-int
-Type(v)
-	void *v;
+int 
+Type(void *v)
 {
 	int *msgvec = v;
 
-	return(type1(msgvec, 0, 0, 0, NULL, NULL));
+	return(type1(msgvec, 0, 0, 0, 0, NULL, NULL));
+}
+
+/*
+ * Show MIME-encoded message text, including all fields.
+ */
+int
+show(void *v)
+{
+	int *msgvec = v;
+
+	return(type1(msgvec, 0, 0, 0, 1, NULL, NULL));
 }
 
 /*
  * Pipe messages, honor ignored fields.
  */
-int
-pipecmd(v)
-void *v;
+int 
+pipecmd(void *v)
 {
 	char *str = v;
 	return(pipe1(str, 1));
@@ -679,9 +1006,8 @@ void *v;
 /*
  * Pipe messages, not respecting ignored fields.
  */
-int
-Pipecmd(v)
-void *v;
+int 
+Pipecmd(void *v)
 {
 	char *str = v;
 	return(pipe1(str, 0));
@@ -692,9 +1018,8 @@ void *v;
  * probably caused by quitting more.
  */
 /*ARGSUSED*/
-static RETSIGTYPE
-brokpipe(signo)
-	int signo;
+void
+brokpipe(int signo)
 {
 	siglongjmp(pipestop, 1);
 }
@@ -704,9 +1029,8 @@ brokpipe(signo)
  * The number of lines is taken from the variable "toplines"
  * and defaults to 5.
  */
-int
-top(v)
-	void *v;
+int 
+top(void *v)
 {
 	int *msgvec = v;
 	int *ip;
@@ -724,7 +1048,7 @@ top(v)
 			topl = 5;
 	}
 	lineb = 1;
-	for (ip = msgvec; *ip && ip-msgvec < msgcount; ip++) {
+	for (ip = msgvec; *ip && ip-msgvec < msgCount; ip++) {
 		mp = &message[*ip - 1];
 		touch(mp);
 		setdot(mp);
@@ -735,7 +1059,7 @@ top(v)
 		if (mp->m_flag & MNOFROM)
 			printf("From %s %s\n", fakefrom(mp),
 					fakedate(mp->m_time));
-		if ((ibuf = setinput(mp, NEED_BODY)) == NULL)	/* XXX could use TOP */
+		if ((ibuf = setinput(&mb, mp, NEED_BODY)) == NULL)	/* XXX could use TOP */
 			return 1;
 		c = mp->m_lines;
 		if (!lineb)
@@ -756,9 +1080,8 @@ top(v)
  * Touch all the given messages so that they will
  * get mboxed.
  */
-int
-stouch(v)
-	void *v;
+int 
+stouch(void *v)
 {
 	int *msgvec = v;
 	int *ip;
@@ -767,6 +1090,10 @@ stouch(v)
 		setdot(&message[*ip-1]);
 		dot->m_flag |= MTOUCH;
 		dot->m_flag &= ~MPRESERVE;
+		/*
+		 * POSIX interpretation necessary.
+		 */
+		did_print_dot = 1;
 	}
 	return(0);
 }
@@ -774,9 +1101,8 @@ stouch(v)
 /*
  * Make sure all passed messages get mboxed.
  */
-int
-mboxit(v)
-	void *v;
+int 
+mboxit(void *v)
 {
 	int *msgvec = v;
 	int *ip;
@@ -785,6 +1111,10 @@ mboxit(v)
 		setdot(&message[*ip-1]);
 		dot->m_flag |= MTOUCH|MBOX;
 		dot->m_flag &= ~MPRESERVE;
+		/*
+		 * POSIX interpretation necessary.
+		 */
+		did_print_dot = 1;
 	}
 	return(0);
 }
@@ -792,21 +1122,27 @@ mboxit(v)
 /*
  * List the folders the user currently has.
  */
-/*ARGSUSED*/
-int
-folders(v)
-	void *v;
+int 
+folders(void *v)
 {
+	char	**argv = v;
 	char dirname[PATHSIZE];
-	char *cmd;
+	char *cmd, *name;
 
-	if (getfold(dirname, sizeof dirname) < 0) {
+	if (*argv)
+		name = expand(*argv);
+	else if (getfold(dirname, sizeof dirname) < 0) {
 		printf(catgets(catd, CATSET, 20,
 				"No value set for \"folder\"\n"));
 		return 1;
+	} else
+		name = dirname;
+	if (which_protocol(name) == PROTO_IMAP)
+		imap_folders(name, *argv == NULL);
+	else {
+		if ((cmd = value("LISTER")) == NULL)
+			cmd = "ls";
+		run_command(cmd, 0, -1, -1, name, NULL, NULL);
 	}
-	if ((cmd = value("LISTER")) == NULL)
-		cmd = "ls";
-	(void) run_command(cmd, 0, -1, -1, dirname, NULL, NULL);
 	return 0;
 }
