@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * Copyright (c) 1980, 1993
@@ -38,35 +38,31 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)cmd2.c	2.8 (gritter) 11/20/02";
+static char sccsid[] = "@(#)cmd2.c	2.43 (gritter) 12/2/04";
 #endif
 #endif /* not lint */
 
 #include "rcv.h"
-#ifdef	HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
 #include "extern.h"
-#ifdef	HAVE_STRINGS_H
-#include <strings.h>
-#endif
+#include <sys/wait.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 /*
  * Mail -- a mail program
  *
  * More user commands.
  */
-static int	igcomp __P((const void *, const void *));
-static int	save1 __P((char [], int, char *, struct ignoretab *, int, int));
-static char	*snarf __P((char [], int *, int));
-static int	delm __P((int []));
-#ifdef	DEBUG_COMMANDS
-static void	clob1 __P((int));
-#endif
-static int	ignore1 __P((char *[], struct ignoretab *, char *));
-static int	igshow __P((struct ignoretab *, char *));
-static void	unignore_one __P((const char *, struct ignoretab *));
-static int	unignore1 __P((char *[], struct ignoretab *, char *));
+
+static int save1(char *str, int mark, char *cmd, struct ignoretab *ignore,
+		int convert, int sender_record, int domove);
+static char *snarf(char *linebuf, int *flag, int usembox);
+static int delm(int *msgvec);
+static int ignore1(char **list, struct ignoretab *tab, char *which);
+static int igshow(struct ignoretab *tab, char *which);
+static int igcomp(const void *l, const void *r);
+static void unignore_one(const char *name, struct ignoretab *tab);
+static int unignore1(char **list, struct ignoretab *tab, char *which);
 
 /*
  * If any arguments were given, go to the next applicable argument
@@ -74,8 +70,7 @@ static int	unignore1 __P((char *[], struct ignoretab *, char *));
  * If given as first command with no arguments, print first message.
  */
 int
-next(v)
-	void *v;
+next(void *v)
 {
 	int *msgvec = v;
 	struct message *mp;
@@ -97,15 +92,25 @@ next(v)
 		 * message list which follows dot.
 		 */
 
-		for (ip = msgvec; *ip != 0; ip++)
-			if (*ip > mdot)
+		for (ip = msgvec; *ip != 0; ip++) {
+#ifdef	_CRAY
+/*
+ * Work around an optimizer bug in Cray Standard C Version 4.0.3  (057126).
+ * Otherwise, SIGFPE is received when mb.mb_threaded != 0.
+ */
+#pragma _CRI suppress ip
+#endif	/* _CRAY */
+			if (mb.mb_threaded ? message[*ip-1].m_threadpos >
+						dot->m_threadpos :
+					*ip > mdot)
 				break;
+		}
 		if (*ip == 0)
 			ip = msgvec;
 		ip2 = ip;
 		do {
 			mp = &message[*ip2 - 1];
-			if ((mp->m_flag & MDELETED) == 0) {
+			if ((mp->m_flag & (MDELETED|MHIDDEN)) == 0) {
 				setdot(mp);
 				goto hitit;
 			}
@@ -124,7 +129,7 @@ next(v)
 	 */
 
 	if (!sawcom) {
-		if (msgcount == 0)
+		if (msgCount == 0)
 			goto ateof;
 		goto hitit;
 	}
@@ -134,10 +139,18 @@ next(v)
 	 * wraparound.
 	 */
 
-	for (mp = dot + did_print_dot; mp < &message[msgcount]; mp++)
-		if ((mp->m_flag & (MDELETED|MSAVED)) == 0)
-			break;
-	if (mp >= &message[msgcount]) {
+	if (mb.mb_threaded == 0) {
+		for (mp = dot + did_print_dot; mp < &message[msgCount]; mp++)
+			if ((mp->m_flag & (MDELETED|MSAVED|MHIDDEN|MKILL)) == 0)
+				break;
+	} else {
+		mp = dot;
+		if (did_print_dot)
+			mp = next_in_thread(mp);
+		while (mp && mp->m_flag & (MDELETED|MSAVED|MHIDDEN|MKILL))
+			mp = next_in_thread(mp);
+	}
+	if (mp == NULL || mp >= &message[msgCount]) {
 ateof:
 		printf(catgets(catd, CATSET, 22, "At EOF\n"));
 		return(0);
@@ -157,77 +170,114 @@ hitit:
  * Save a message in a file.  Mark the message as saved
  * so we can discard when the user quits.
  */
-int
-save(v)
-	void *v;
+int 
+save(void *v)
 {
 	char *str = v;
 
-	return save1(str, 1, "save", saveignore, CONV_NONE, 0);
+	return save1(str, 1, "save", saveignore, SEND_MBOX, 0, 0);
 }
 
-int
-Save(v)
-	void *v;
+int 
+Save(void *v)
 {
 	char *str = v;
 
-	return save1(str, 1, "save", saveignore, CONV_NONE, 1);
+	return save1(str, 1, "save", saveignore, SEND_MBOX, 1, 0);
 }
 
 /*
  * Copy a message to a file without affected its saved-ness
  */
-int
-copycmd(v)
-	void *v;
+int 
+copycmd(void *v)
 {
 	char *str = v;
 
-	return save1(str, 0, "copy", saveignore, CONV_NONE, 0);
+	return save1(str, 0, "copy", saveignore, SEND_MBOX, 0, 0);
 }
 
-int
-Copycmd(v)
-	void *v;
+int 
+Copycmd(void *v)
 {
 	char *str = v;
 
-	return save1(str, 0, "copy", saveignore, CONV_NONE, 1);
+	return save1(str, 0, "copy", saveignore, SEND_MBOX, 1, 0);
+}
+
+/*
+ * Move a message to a file.
+ */
+int 
+cmove(void *v)
+{
+	char *str = v;
+
+	return save1(str, 0, "move", saveignore, SEND_MBOX, 0, 1);
+}
+
+int 
+cMove(void *v)
+{
+	char *str = v;
+
+	return save1(str, 0, "move", saveignore, SEND_MBOX, 1, 1);
+}
+
+/*
+ * Decrypt and copy a message to a file.
+ */
+int 
+cdecrypt(void *v)
+{
+	char *str = v;
+
+	return save1(str, 0, "decrypt", saveignore, SEND_DECRYPT, 0, 0);
+}
+
+int 
+cDecrypt(void *v)
+{
+	char *str = v;
+
+	return save1(str, 0, "decrypt", saveignore, SEND_DECRYPT, 1, 0);
 }
 
 /*
  * Save/copy the indicated messages at the end of the passed file name.
  * If mark is true, mark the message "saved."
  */
-static int
-save1(str, mark, cmd, ignore, convert, sender_record)
-	char str[];
-	int mark;
-	char *cmd;
-	struct ignoretab *ignore;
+static int 
+save1(char *str, int mark, char *cmd, struct ignoretab *ignore,
+		int convert, int sender_record, int domove)
 {
+	struct stat	st;
 	int *ip;
 	struct message *mp;
-	char *file = NULL, *disp;
+	char *file = NULL, *disp = "";
 	int f, *msgvec;
 	FILE *obuf;
-	int newfile;
+	int newfile = 0;
 	char *cp, *cq;
 	off_t mstats[2], tstats[2];
+	int compressed = 0;
+	enum protocol prot;
+	int success = 1, last = 0;
 
 	/*LINTED*/
-	msgvec = (int *)salloc((msgcount + 2) * sizeof *msgvec);
+	msgvec = (int *)salloc((msgCount + 2) * sizeof *msgvec);
 	if (sender_record) {
 		for (cp = str; *cp && blankchar(*cp & 0377); cp++);
 		f = (*cp != '\0');
 	} else {
-		if ((file = snarf(str, &f, convert != CONV_TOFILE)) == NULL)
+		if ((file = snarf(str, &f, convert != SEND_TOFILE)) == NULL)
 			return(1);
 	}
 	if (!f) {
 		*msgvec = first(0, MMNORM);
 		if (*msgvec == 0) {
+			if (inhook)
+				return 0;
 			printf(catgets(catd, CATSET, 23,
 					"No messages to %s.\n"), cmd);
 			return(1);
@@ -236,6 +286,12 @@ save1(str, mark, cmd, ignore, convert, sender_record)
 	}
 	if (f && getmsglist(str, msgvec, 0) < 0)
 		return(1);
+	if (*msgvec == 0) {
+		if (inhook)
+			return 0;
+		printf("No applicable messages.\n");
+		return 1;
+	}
 	if (sender_record) {
 		if ((cp = nameof(&message[*msgvec - 1], 0)) == NULL) {
 			printf(catgets(catd, CATSET, 24,
@@ -254,20 +310,31 @@ save1(str, mark, cmd, ignore, convert, sender_record)
 	}
 	if ((file = expand(file)) == NULL)
 		return(1);
-	if (access(file, 0) >= 0) {
-		newfile = 0;
-		disp = catgets(catd, CATSET, 25, "[Appended]");
-	} else {
-		newfile = 1;
-		disp = catgets(catd, CATSET, 26, "[New file]");
+	prot = which_protocol(file);
+	if (prot != PROTO_IMAP) {
+		if (access(file, 0) >= 0) {
+			newfile = 0;
+			disp = catgets(catd, CATSET, 25, "[Appended]");
+		} else {
+			newfile = 1;
+			disp = catgets(catd, CATSET, 26, "[New file]");
+		}
 	}
-	if ((obuf = Fopen(file, "a+")) == NULL) {
-		if ((obuf = Fopen(file, "wx")) == NULL) {
+	if ((obuf = convert == SEND_TOFILE ? Fopen(file, "a+") :
+			Zopen(file, "a+", &compressed)) == NULL) {
+		if ((obuf = convert == SEND_TOFILE ? Fopen(file, "wx") :
+				Zopen(file, "wx", &compressed)) == NULL) {
 			perror(file);
 			return(1);
 		}
 	} else {
-		if (!newfile && fseek(obuf, -2L, SEEK_END) == 0) {
+		if (compressed) {
+			newfile = 0;
+			disp = catgets(catd, CATSET, 25, "[Appended]");
+		}
+		if (!newfile && fstat(fileno(obuf), &st) &&
+				S_ISREG(st.st_mode) &&
+				fseek(obuf, -2L, SEEK_END) == 0) {
 			char buf[2];
 			int prependnl = 0;
 
@@ -291,15 +358,27 @@ save1(str, mark, cmd, ignore, convert, sender_record)
 			}
 			fflush(obuf);
 			if (prependnl) {
-				sputc('\n', obuf);
+				putc('\n', obuf);
 				fflush(obuf);
 			}
 		}
 	}
 	tstats[0] = tstats[1] = 0;
-	for (ip = msgvec; *ip && ip-msgvec < msgcount; ip++) {
+	imap_created_mailbox = 0;
+	for (ip = msgvec; *ip && ip-msgvec < msgCount; ip++) {
 		mp = &message[*ip - 1];
-		if (send_message(mp, obuf, ignore, NULL, convert, mstats) < 0) {
+		if (prot == PROTO_IMAP &&
+				ignore[0].i_count == 0 &&
+				ignore[1].i_count == 0 &&
+				imap_thisaccount(file)) {
+			if (imap_copy(mp, *ip, file) == STOP) {
+				Fclose(obuf);
+				return 1;
+			}
+			mstats[0] = -1;
+			mstats[1] = mp->m_xsize;
+		} else if (send(mp, obuf, ignore, NULL,
+					convert, mstats) < 0) {
 			perror(file);
 			Fclose(obuf);
 			return(1);
@@ -307,20 +386,42 @@ save1(str, mark, cmd, ignore, convert, sender_record)
 		touch(mp);
 		if (mark)
 			mp->m_flag |= MSAVED;
+		if (domove) {
+			mp->m_flag |= MDELETED|MSAVED;
+			last = *ip;
+		}
 		tstats[0] += mstats[0];
 		tstats[1] += mstats[1];
 	}
 	fflush(obuf);
-	if (ferror(obuf))
+	if (ferror(obuf)) {
 		perror(file);
-	Fclose(obuf);
-	printf("\"%s\" %s ", file, disp);
-	if (tstats[0] >= 0)
-		printf("%lu", (long)tstats[0]);
-	else
-		printf(catgets(catd, CATSET, 27, "binary"));
-	printf("/%lu\n", (long)tstats[1]);
-	return(0);
+		success = 0;
+	}
+	if (Fclose(obuf) != 0)
+		success = 0;
+	if (success) {
+		if (prot == PROTO_IMAP || prot == PROTO_MAILDIR)
+			disp = prot == PROTO_IMAP && disconnected(file) ?
+				"[Queued]" : imap_created_mailbox ?
+					"[New file]" : "[Appended]";
+		printf("\"%s\" %s ", file, disp);
+		if (tstats[0] >= 0)
+			printf("%lu", (long)tstats[0]);
+		else
+			printf(catgets(catd, CATSET, 27, "binary"));
+		printf("/%lu\n", (long)tstats[1]);
+	} else if (mark)
+		for (ip = msgvec; *ip && ip-msgvec < msgCount; ip++) {
+			mp = &message[*ip - 1];
+			mp->m_flag &= ~MSAVED;
+		}
+	if (domove && last) {
+		setdot(&message[last-1]);
+		last = first(0, MDELETED);
+		setdot(&message[last ? last-1 : 0]);
+	}
+	return(success == 0);
 }
 
 /*
@@ -328,13 +429,12 @@ save1(str, mark, cmd, ignore, convert, sender_record)
  * file name, minus header and trailing blank line.
  * This is the MIME save function.
  */
-int
-swrite(v)
-	void *v;
+int 
+cwrite(void *v)
 {
 	char *str = v;
 
-	return save1(str, 0, "write", allignore, CONV_TOFILE, 0);
+	return save1(str, 0, "write", allignore, SEND_TOFILE, 0, 0);
 }
 
 /*
@@ -347,9 +447,7 @@ swrite(v)
  */
 
 static char *
-snarf(linebuf, flag, usembox)
-	char linebuf[];
-	int *flag;
+snarf(char *linebuf, int *flag, int usembox)
 {
 	char *cp;
 
@@ -370,9 +468,8 @@ snarf(linebuf, flag, usembox)
 /*
  * Delete messages.
  */
-int
-delete(v)
-	void *v;
+int 
+delete(void *v)
 {
 	int *msgvec = v;
 	delm(msgvec);
@@ -382,9 +479,8 @@ delete(v)
 /*
  * Delete messages, then type the new dot.
  */
-int
-deltype(v)
-	void *v;
+int 
+deltype(void *v)
 {
 	int *msgvec = v;
 	int list[2];
@@ -409,9 +505,8 @@ deltype(v)
  * Set dot to some nice place afterwards.
  * Internal interface.
  */
-static int
-delm(msgvec)
-	int *msgvec;
+static int 
+delm(int *msgvec)
 {
 	struct message *mp;
 	int *ip;
@@ -448,19 +543,20 @@ delm(msgvec)
 /*
  * Undelete the indicated messages.
  */
-int
-undeletecmd(v)
-	void *v;
+int 
+undeletecmd(void *v)
 {
 	int *msgvec = v;
 	struct message *mp;
 	int *ip;
 
-	for (ip = msgvec; *ip && ip-msgvec < msgcount; ip++) {
+	for (ip = msgvec; *ip && ip-msgvec < msgCount; ip++) {
 		mp = &message[*ip - 1];
 		touch(mp);
 		setdot(mp);
 		mp->m_flag &= ~MDELETED;
+		if (mb.mb_type == MB_IMAP || mb.mb_type == MB_CACHE)
+			imap_undelete(mp, *ip);
 	}
 	return 0;
 }
@@ -470,9 +566,8 @@ undeletecmd(v)
  * Interactively dump core on "core"
  */
 /*ARGSUSED*/
-int
-core(v)
-	void *v;
+int 
+core(void *v)
 {
 	int pid;
 #ifdef	WCOREDUMP
@@ -502,9 +597,8 @@ core(v)
 /*
  * Clobber as many bytes of stack as the user requests.
  */
-int
-clobber(v)
-	void *v;
+int 
+clobber(void *v)
 {
 	char **argv = v;
 	int times;
@@ -520,9 +614,8 @@ clobber(v)
 /*
  * Clobber the stack.
  */
-static void
-clob1(n)
-	int n;
+static void 
+clob1(int n)
 {
 	char buf[512];
 	char *cp;
@@ -539,9 +632,8 @@ clob1(n)
  * Add the given header fields to the retained list.
  * If no arguments, print the current list of retained fields.
  */
-int
-retfield(v)
-	void *v;
+int 
+retfield(void *v)
 {
 	char **list = v;
 
@@ -552,38 +644,48 @@ retfield(v)
  * Add the given header fields to the ignored list.
  * If no arguments, print the current list of ignored fields.
  */
-int
-igfield(v)
-	void *v;
+int 
+igfield(void *v)
 {
 	char **list = v;
 
 	return ignore1(list, ignore, "ignored");
 }
 
-int
-saveretfield(v)
-	void *v;
+int 
+saveretfield(void *v)
 {
 	char **list = v;
 
 	return ignore1(list, saveignore + 1, "retained");
 }
 
-int
-saveigfield(v)
-	void *v;
+int 
+saveigfield(void *v)
 {
 	char **list = v;
 
 	return ignore1(list, saveignore, "ignored");
 }
 
-static int
-ignore1(list, tab, which)
-	char *list[];
-	struct ignoretab *tab;
-	char *which;
+int 
+fwdretfield(void *v)
+{
+	char **list = v;
+
+	return ignore1(list, fwdignore + 1, "retained");
+}
+
+int 
+fwdigfield(void *v)
+{
+	char **list = v;
+
+	return ignore1(list, fwdignore, "ignored");
+}
+
+static int 
+ignore1(char **list, struct ignoretab *tab, char *which)
 {
 	int h;
 	struct ignore *igp;
@@ -618,10 +720,8 @@ ignore1(list, tab, which)
 /*
  * Print out all currently retained fields.
  */
-static int
-igshow(tab, which)
-	struct ignoretab *tab;
-	char *which;
+static int 
+igshow(struct ignoretab *tab, char *which)
 {
 	int h;
 	struct ignore *igp;
@@ -648,45 +748,50 @@ igshow(tab, which)
 /*
  * Compare two names for sorting ignored field list.
  */
-static int
-igcomp(l, r)
-	const void *l, *r;
+static int 
+igcomp(const void *l, const void *r)
 {
 	return (strcmp(*(char **)l, *(char **)r));
 }
 
-int
-unignore(v)
-	void *v;
+int 
+unignore(void *v)
 {
 	return unignore1((char **)v, ignore, "ignored");
 }
 
-int
-unretain(v)
-	void *v;
+int 
+unretain(void *v)
 {
 	return unignore1((char **)v, ignore + 1, "retained");
 }
 
-int
-unsaveignore(v)
-	void *v;
+int 
+unsaveignore(void *v)
 {
 	return unignore1((char **)v, saveignore, "ignored");
 }
 
-int
-unsaveretain(v)
-	void *v;
+int 
+unsaveretain(void *v)
 {
 	return unignore1((char **)v, saveignore + 1, "retained");
 }
 
-static void
-unignore_one(name, tab)
-	const char *name;
-	struct ignoretab *tab;
+int 
+unfwdignore(void *v)
+{
+	return unignore1((char **)v, fwdignore, "ignored");
+}
+
+int 
+unfwdretain(void *v)
+{
+	return unignore1((char **)v, fwdignore + 1, "retained");
+}
+
+static void 
+unignore_one(const char *name, struct ignoretab *tab)
 {
 	struct ignore *ip, *iq = NULL;
 	int h = hash(name);
@@ -705,11 +810,8 @@ unignore_one(name, tab)
 	}
 }
 
-static int
-unignore1(list, tab, which)
-	char *list[];
-	struct ignoretab *tab;
-	char *which;
+static int 
+unignore1(char **list, struct ignoretab *tab, char *which)
 {
 	if (tab->i_count == 0) {
 		printf(catgets(catd, CATSET, 34,
