@@ -38,7 +38,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)cmd3.c	2.76 (gritter) 12/3/04";
+static char sccsid[] = "@(#)cmd3.c	2.80 (gritter) 2/12/05";
 #endif
 #endif /* not lint */
 
@@ -55,7 +55,7 @@ static char sccsid[] = "@(#)cmd3.c	2.76 (gritter) 12/3/04";
  */
 
 static int bangexp(char **str, size_t *size);
-static void make_ref(struct message *mp, struct header *head);
+static void make_ref_and_cs(struct message *mp, struct header *head);
 static int (*respond_or_Respond(int c))(int *, int);
 static int respond_internal(int *msgvec, int recipient_record);
 static char *reedit(char *subj);
@@ -223,9 +223,9 @@ schdir(void *v)
 }
 
 static void 
-make_ref(struct message *mp, struct header *head)
+make_ref_and_cs(struct message *mp, struct header *head)
 {
-	char *oldref, *oldmsgid, *newref;
+	char *oldref, *oldmsgid, *newref, *cp;
 	size_t reflen;
 	unsigned i;
 	struct name *n;
@@ -265,6 +265,9 @@ make_ref(struct message *mp, struct header *head)
 	}
 	n->n_blink = NULL;
 	head->h_ref = n;
+	if (value("reply-in-same-charset") != NULL &&
+			(cp = hfield("content-type", mp)) != NULL)
+		head->h_charset = mime_getparam("charset", cp);
 }
 
 static int
@@ -359,9 +362,9 @@ respond_internal(int *msgvec, int recipient_record)
 		np = delete_alternates(np);
 		head.h_cc = np;
 	}
-	make_ref(mp, &head);
+	make_ref_and_cs(mp, &head);
 	if (mail1(&head, 1, mp, NULL, recipient_record, 0, 0) == OKAY &&
-			value("markanswered"))
+			value("markanswered") && (mp->m_flag & MANSWERED) == 0)
 		mp->m_flag |= MANSWER|MANSWERED;
 	return(0);
 }
@@ -401,7 +404,9 @@ forward1(char *str, int recipient_record)
 	char	*recipient;
 	struct message	*mp;
 	struct header	head;
+	int	forward_as_attachment;
 
+	forward_as_attachment = value("forward-as-attachment") != NULL;
 	msgvec = salloc((msgCount + 2) * sizeof *msgvec);
 	if ((recipient = laststring(str, &f, 0)) == NULL) {
 		puts(catgets(catd, CATSET, 47, "No recipient specified."));
@@ -429,17 +434,23 @@ forward1(char *str, int recipient_record)
 		printf("Cannot forward multiple messages at once\n");
 		return 1;
 	}
-	mp = &message[*msgvec - 1];
-	touch(mp);
-	setdot(mp);
 	memset(&head, 0, sizeof head);
 	if ((head.h_to = sextract(recipient,
 			GTO | (value("fullnames") ? GFULL : GSKIN))) == NULL)
 		return 1;
+	mp = &message[*msgvec - 1];
+	if (forward_as_attachment) {
+		head.h_attach = csalloc(1, sizeof *head.h_attach);
+		head.h_attach->a_msgno = *msgvec;
+	} else {
+		touch(mp);
+		setdot(mp);
+	}
 	if ((head.h_subject = hfield("subject", mp)) == NULL)
 		head.h_subject = hfield("subj", mp);
 	head.h_subject = fwdedit(head.h_subject);
-	mail1(&head, 1, mp, NULL, recipient_record, 1, 0);
+	mail1(&head, 1, forward_as_attachment ? NULL : mp,
+			NULL, recipient_record, 1, 0);
 	return 0;
 }
 
@@ -505,6 +516,9 @@ preserve(void *v)
 		mp->m_flag |= MPRESERVE;
 		mp->m_flag &= ~MBOX;
 		setdot(mp);
+		/*
+		 * This is now Austin Group Request XCU #20.
+		 */
 		did_print_dot = 1;
 	}
 	return(0);
@@ -525,6 +539,10 @@ unread(void *v)
 		dot->m_flag |= MSTATUS;
 		if (mb.mb_type == MB_IMAP || mb.mb_type == MB_CACHE)
 			imap_unread(&message[*ip-1], *ip);
+		/*
+		 * The "unread" command is not part of POSIX mailx.
+		 */
+		did_print_dot = 1;
 	}
 	return(0);
 }
@@ -956,9 +974,9 @@ Respond_internal(int *msgvec, int recipient_record)
 	if ((head.h_subject = hfield("subject", mp)) == NULL)
 		head.h_subject = hfield("subj", mp);
 	head.h_subject = reedit(head.h_subject);
-	make_ref(mp, &head);
+	make_ref_and_cs(mp, &head);
 	if (mail1(&head, 1, mp, NULL, recipient_record, 0, 0) == OKAY &&
-			value("markanswered"))
+			value("markanswered") && (mp->m_flag & MANSWERED) == 0)
 		mp->m_flag |= MANSWER|MANSWERED;
 	return 0;
 }
