@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * Copyright (c) 1996 Christos Zoulas.  All rights reserved.
@@ -34,16 +34,16 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)dotlock.c	2.1 (gritter) 9/1/02";
+static char sccsid[] = "@(#)dotlock.c	2.7 (gritter) 10/2/04";
 #endif
 #endif
 
 #include "rcv.h"
-#include <sys/types.h>
-#ifdef	HAVE_SYS_UTSNAME_H
+#include <sys/stat.h>
+#include <unistd.h>
+#include <fcntl.h>
+#include <time.h>
 #include <sys/utsname.h>
-#endif
-
 #include <errno.h>
 
 #include "extern.h"
@@ -52,13 +52,13 @@ static char sccsid[] = "@(#)dotlock.c	2.1 (gritter) 9/1/02";
 #define O_SYNC	0
 #endif
 
-static int	perhaps_setgid __P((const char *name, gid_t gid));
-static int	maildir_access __P((const char *));
+static int maildir_access(const char *fname);
+static int perhaps_setgid(const char *name, gid_t gid);
+static int create_exclusive(const char *fname);
 
 /* Check if we can write a lock file at all */
-static int
-maildir_access(fname)
-const char *fname;
+static int 
+maildir_access(const char *fname)
 {
 	char path[MAXPATHLEN];
 	char *p;
@@ -76,11 +76,9 @@ const char *fname;
  * Set the gid if the path is in the normal mail spool
  */
 static int
-perhaps_setgid(name, gid)
-const char *name;
-gid_t gid;
+perhaps_setgid(const char *name, gid_t gid)
 {
-	char safepath[]= PATH_MAILDIR;
+	char safepath[]= MAILSPOOL;
 
 	if (strncmp(name, safepath, sizeof (safepath)-1) ||
 			strchr(name + sizeof (safepath), '/'))
@@ -89,8 +87,6 @@ gid_t gid;
 }
 
 
-static int create_exclusive __P((const char *));
-
 #define	APID_SZ	40	/* sufficient for storign 128 bits pids */
 /*
  * Create a unique file. O_EXCL does not really work over NFS so we follow
@@ -102,9 +98,8 @@ static int create_exclusive __P((const char *));
  * - unlink the mostly unique filename
  * - if the link count was 2, then we are ok; else we've failed.
  */
-static int
-create_exclusive(fname)
-	const char *fname;
+static int 
+create_exclusive(const char *fname)
 {
 	char path[MAXPATHLEN];
 	char *hostname;
@@ -115,22 +110,11 @@ create_exclusive(fname)
 	size_t ntries, cookie;
 	int fd, serrno, cc;
 	struct stat st;
-#if defined (HAVE_UNAME)
 	struct utsname ut;
-#elif defined (HAVE_GETHOSTNAME)
-	char host__name[MAXHOSTNAMELEN];
-#endif
 
 	time(&t);
-#if defined (HAVE_UNAME)
 	uname(&ut);
 	hostname = ut.nodename;
-#elif defined (HAVE_GETHOSTNAME)
-	gethostname(host__name, MAXHOSTNAMELEN);
-	hostname = host__name;
-#else
-	hostname = "unknown";
-#endif
 	pid = getpid();
 
 	cookie = (int)pid ^ (int)t;
@@ -143,7 +127,7 @@ create_exclusive(fname)
 	else
 		ptr++;
 
-	(void) snprintf(path, sizeof path, "%.*s.%s.%x", 
+	snprintf(path, sizeof path, "%.*s.%s.%x", 
 	    (int) (ptr - fname), fname, hostname, (unsigned int) cookie);
 
 	/*
@@ -156,7 +140,7 @@ create_exclusive(fname)
 		if (fd != -1) {
 			snprintf(apid, APID_SZ, "%d", (int)getpid());
 			write(fd, apid, strlen(apid));
-			(void) close(fd);
+			close(fd);
 			break;
 		}
 		else if (errno == EEXIST)
@@ -182,7 +166,7 @@ create_exclusive(fname)
 		goto bad;
 
 	perhaps_setgid(fname, effectivegid);
-	(void) unlink(path);
+	unlink(path);
 	setgid(realgid);
 
 	/*
@@ -197,15 +181,14 @@ create_exclusive(fname)
 
 bad:
 	serrno = errno;
-	(void) unlink(path);
+	unlink(path);
 	errno = serrno;
 	return -1;
 }
 
-int
-fcntl_lock(fd, type)
+int 
+fcntl_lock(int fd, int type)
 {
-#ifdef	F_SETLKW
 	struct flock flp;
 
 	flp.l_type = type;
@@ -213,18 +196,17 @@ fcntl_lock(fd, type)
 	flp.l_whence = SEEK_SET;
 	flp.l_len = 0;
 	return fcntl(fd, F_SETLKW, &flp);
-#else	/* !F_SETLKW */
-	return flock(fd, type);
-#endif	/* !F_SETLKW */
 }
 
 int
-dot_lock(fname, fd, pollinterval, fp, msg)
+dot_lock(const char *fname, int fd, int pollinterval, FILE *fp, const char *msg)
+#ifdef	notdef
 	const char *fname;	/* Pathname to lock */
 	int fd;			/* File descriptor for fname, for fcntl lock */
 	int pollinterval;	/* Interval to check for lock, -1 return */
 	FILE *fp;		/* File to print message */
 	const char *msg;	/* Message to print */
+#endif
 {
 	char path[MAXPATHLEN];
 	sigset_t nset, oset;
@@ -243,29 +225,25 @@ dot_lock(fname, fd, pollinterval, fp, msg)
 	sigaddset(&nset, SIGTSTP);
 	sigaddset(&nset, SIGCHLD);
 
-	(void) snprintf(path, sizeof(path), "%s.lock", fname);
+	snprintf(path, sizeof(path), "%s.lock", fname);
 
 	for (i=0;i<15;i++) {
-		(void) sigprocmask(SIG_BLOCK, &nset, &oset);
+		sigprocmask(SIG_BLOCK, &nset, &oset);
 		if (create_exclusive(path) != -1) {
-			(void) sigprocmask(SIG_SETMASK, &oset, NULL);
+			sigprocmask(SIG_SETMASK, &oset, NULL);
 			return 0;
 		}
 		else {
 			olderrno = errno;
-			(void) sigprocmask(SIG_SETMASK, &oset, NULL);
+			sigprocmask(SIG_SETMASK, &oset, NULL);
 		}
 
-#ifdef	F_SETLKW
 		fcntl_lock(fd, F_UNLCK);
-#else
-		fcntl_lock(fd, LOCK_UN);
-#endif
 		if (olderrno != EEXIST)
 			return -1;
 
 		if (fp && msg)
-		    (void) fputs(msg, fp);
+		    fputs(msg, fp);
 
 		if (pollinterval) {
 			if (pollinterval == -1) {
@@ -274,28 +252,23 @@ dot_lock(fname, fd, pollinterval, fp, msg)
 			}
 			sleep(pollinterval);
 		}
-#ifdef	F_SETLKW
 		fcntl_lock(fd, F_WRLCK);
-#else
-		fcntl_lock(fd, LOCK_EX);
-#endif
 	}
         fprintf(stderr, catgets(catd, CATSET, 71,
 		"%s seems a stale lock? Need to be removed by hand?\n"), path);
         return -1;
 }
 
-void
-dot_unlock(fname)
-	const char *fname;
+void 
+dot_unlock(const char *fname)
 {
 	char path[MAXPATHLEN];
 
 	if (maildir_access(fname) != 0)
 		return;
 
-	(void) snprintf(path, sizeof(path), "%s.lock", fname);
+	snprintf(path, sizeof(path), "%s.lock", fname);
 	perhaps_setgid(path, effectivegid);
-	(void) unlink(path);
+	unlink(path);
 	setgid(realgid);
 }
