@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * Copyright (c) 1980, 1993
@@ -40,7 +40,7 @@
 #ifdef	DOSCCS
 static char copyright[]
 = "@(#) Copyright (c) 1980, 1993 The Regents of the University of California.  All rights reserved.\n";
-static char sccsid[] = "@(#)main.c	2.15 (gritter) 2/27/03";
+static char sccsid[] = "@(#)main.c	2.44 (gritter) 3/4/05";
 #endif	/* DOSCCS */
 #endif /* not lint */
 
@@ -55,12 +55,21 @@ static char sccsid[] = "@(#)main.c	2.15 (gritter) 2/27/03";
  */
 
 
+#include "config.h"
+#ifdef	HAVE_NL_LANGINFO
+#include <langinfo.h>
+#endif	/* HAVE_NL_LANGINFO */
 #define _MAIL_GLOBS_
 #include "rcv.h"
-#ifdef	HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
-#endif
 #include "extern.h"
+#include <sys/stat.h>
+#include <sys/ioctl.h>
+#include <termios.h>
+#include <unistd.h>
+#include <fcntl.h>
+#ifdef	HAVE_SETLOCALE
+#include <locale.h>
+#endif	/* HAVE_SETLOCALE */
 
 /*
  * Mail -- a mail program
@@ -71,19 +80,17 @@ static char sccsid[] = "@(#)main.c	2.15 (gritter) 2/27/03";
 static sigjmp_buf	hdrjmp;
 char	*progname;
 
-static void	hdrstop __P((int));
-static void	setscreensize __P((void));
+static void hdrstop(int signo);
+static void setscreensize(int dummy);
 
-int
-main(argc, argv)
-	int argc;
-	char *argv[];
+int 
+main(int argc, char *argv[])
 {
-	const char *optstr = "+BHFINVT:a:b:c:defh:inqr:s:tu:v~";
+	const char optstr[] = "A:BHFINVT:a:b:c:dDefh:inqr:s:tu:v~";
 	int i, existonly = 0, headersonly = 0, sendflag = 0;
 	struct name *to, *cc, *bcc, *smopts;
 	struct attachment *attach;
-	char *subject, *cp, *ef, *qf = NULL, *fromaddr = NULL;
+	char *subject, *cp, *ef, *qf = NULL, *fromaddr = NULL, *Aflag = NULL;
 	char nosrc = 0;
 	int Fflag = 0, Nflag = 0, tflag = 0;
 	sighandler_type prevint;
@@ -103,6 +110,7 @@ main(argc, argv)
 		exit(1);
 	}
 
+	starting = 1;
 	progname = strrchr(argv[0], '/');
 	if (progname != NULL)
 		progname++;
@@ -113,7 +121,7 @@ main(argc, argv)
 	 * Figure out whether we are being run interactively,
 	 * start the SIGCHLD catcher, and so forth.
 	 */
-	(void) safe_signal(SIGCHLD, sigchild);
+	safe_signal(SIGCHLD, sigchild);
 	is_a_tty[0] = isatty(0);
 	is_a_tty[1] = isatty(1);
 	if (is_a_tty[0])
@@ -122,12 +130,22 @@ main(argc, argv)
 	assign("save", "");
 #ifdef	HAVE_SETLOCALE
 	setlocale(LC_CTYPE, "");
+	setlocale(LC_COLLATE, "");
 	setlocale(LC_MESSAGES, "");
 	mb_cur_max = MB_CUR_MAX;
 #if defined (HAVE_NL_LANGINFO) && defined (CODESET)
 	if (value("ttycharset") == NULL && (cp = nl_langinfo(CODESET)) != NULL)
 		assign("ttycharset", cp);
-#endif
+#endif	/* HAVE_NL_LANGINFO && CODESET */
+#if defined (HAVE_MBTOWC) && defined (HAVE_WCTYPE_H)
+	if (mb_cur_max > 1) {
+		wchar_t	wc;
+		if (mbtowc(&wc, "\303\266", 2) == 2 && wc == 0xF6 &&
+				mbtowc(&wc, "\342\202\254", 3) == 3 &&
+				wc == 0x20AC)
+			utf8 = 1;
+	}
+#endif	/* HAVE_MBTOWC && HAVE_WCTYPE_H */
 #else	/* !HAVE_SETLOCALE */
 	mb_cur_max = 1;
 #endif	/* !HAVE_SETLOCALE */
@@ -151,15 +169,12 @@ main(argc, argv)
 	 * first of these users.
 	 */
 	ef = NULL;
-	to = NIL;
-	cc = NIL;
-	bcc = NIL;
+	to = NULL;
+	cc = NULL;
+	bcc = NULL;
 	attach = NULL;
-	smopts = NIL;
+	smopts = NULL;
 	subject = NULL;
-#ifndef	__GLIBC__
-	optstr++;
-#endif
 	while ((i = getopt(argc, argv, optstr)) != EOF) {
 		switch (i) {
 		case 'V':
@@ -211,6 +226,9 @@ main(argc, argv)
 		case 'd':
 			debug++;
 			break;
+		case 'D':
+			assign("disconnected", "");
+			break;
 		case 'e':
 			existonly++;
 			break;
@@ -287,14 +305,14 @@ main(argc, argv)
 			/*
 			 * Get Carbon Copy Recipient list
 			 */
-			cc = checkaddrs(cat(cc, extract(optarg, GCC)));
+			cc = checkaddrs(cat(cc, extract(optarg, GCC|GFULL)));
 			sendflag++;
 			break;
 		case 'b':
 			/*
 			 * Get Blind Carbon Copy Recipient list
 			 */
-			bcc = checkaddrs(cat(bcc, extract(optarg, GBCC)));
+			bcc = checkaddrs(cat(bcc, extract(optarg, GBCC|GFULL)));
 			sendflag++;
 			break;
 		case 'h':
@@ -313,13 +331,13 @@ main(argc, argv)
 			sendflag = 1;
 			tflag = 1;
 			break;
+		case 'A':
+			Aflag = optarg;
+			break;
 		case '?':
 usage:
 			fprintf(stderr, catgets(catd, CATSET, 135,
-"Usage: %s [-BFintv~] [-s subject] [-a attachment] [-c cc-addr] [-b bcc-addr] [-r from-addr] [-h hops] to-addr ... [- sendmail-options ...]\n\
-       %s [-BeHiInNv~] [-T name] -f [name]\n\
-       %s [-BeinNv~] [-u user]\n"),
-				progname, progname, progname);
+"Usage: %s -eiIUdFntBDNHV~ -T FILE -u USER -h hops -r address -s SUBJECT -a FILE -q FILE -f FILE -A ACCOUNT -b USERS -c USERS users\n"), progname);
 			exit(2);
 		}
 	}
@@ -327,26 +345,24 @@ usage:
 		if (optind < argc) {
 			if (optind + 1 < argc) {
 				fprintf(stderr, catgets(catd, CATSET, 205,
-					"More than on file given with -f\n"));
+					"More than one file given with -f\n"));
 				goto usage;
 			}
 			ef = argv[optind];
 		}
 	} else {
-		for (i = optind; (argv[i]) && (*argv[i] != '-'); i++)
-			to = checkaddrs(cat(to, extract(argv[i], GTO)));
-		for (; argv[i]; i++)
-			smopts = cat(smopts, nalloc(argv[i], 0));
+		for (i = optind; argv[i]; i++)
+			to = checkaddrs(cat(to, extract(argv[i], GTO|GFULL)));
 	}
 	/*
 	 * Check for inconsistent arguments.
 	 */
-	if (ef != NULL && to != NIL) {
+	if (ef != NULL && to != NULL) {
 		fprintf(stderr, catgets(catd, CATSET, 137,
 			"Cannot give -f and people to send to.\n"));
 		goto usage;
 	}
-	if (sendflag && !tflag && to == NIL) {
+	if (sendflag && !tflag && to == NULL) {
 		fprintf(stderr, catgets(catd, CATSET, 138,
 			"Send options without primary recipient specified.\n"));
 		goto usage;
@@ -357,12 +373,17 @@ usage:
 		goto usage;
 	}
 	tinit();
-	setscreensize();
+	setscreensize(0);
+#ifdef	SIGWINCH
+	if (value("interactive"))
+		if (safe_signal(SIGWINCH, SIG_IGN) != SIG_IGN)
+			safe_signal(SIGWINCH, setscreensize);
+#endif	/* SIGWINCH */
 	input = stdin;
 	rcvmode = !to && !tflag;
 	spreserve();
 	if (!nosrc)
-		load(PATH_MASTER_RC);
+		load(MAILRC);
 	/*
 	 * Expand returns a savestr, but load only uses the file name
 	 * for fopen, so it's safe to do this.
@@ -376,6 +397,18 @@ usage:
 	if (getenv("NAIL_EXTRA_RC") == NULL &&
 			(cp = value("NAIL_EXTRA_RC")) != NULL)
 		load(expand(cp));
+	/*
+	 * Now we can set the account.
+	 */
+	if (Aflag) {
+		char	*a[2];
+		a[0] = Aflag;
+		a[1] = NULL;
+		account(a);
+	}
+
+	starting = 0;
+
 	/*
 	 * From address from command line overrides rc files.
 	 */
@@ -386,7 +419,7 @@ usage:
 		/*
 		 * why wait?
 		 */
-		exit(senderr);
+		exit(senderr ? 1 : 0);
 	}
 	/*
 	 * Ok, we are reading mail.
@@ -395,16 +428,28 @@ usage:
 	 */
 	if (ef == NULL)
 		ef = "%";
+	else if (*ef == '@') {
+		/*
+		 * This must be treated specially to make invocation like
+		 * -A imap -f @mailbox work.
+		 */
+		if ((cp = value("folder")) != NULL &&
+				which_protocol(cp) == PROTO_IMAP)
+			strcpy(mailname, cp);
+	}
 	i = setfile(ef, 0);
 	if (i < 0)
 		exit(1);		/* error already reported */
 	if (existonly)
 		exit(i);
 	if (headersonly) {
-		for (i = 1; i <= msgcount; i++)
-			printhead(i, stdout);
+		if (mb.mb_type == MB_IMAP)
+			imap_getheaders(1, msgCount);
+		for (i = 1; i <= msgCount; i++)
+			printhead(i, stdout, 0);
 		exit(exit_status);
 	}
+	callhook(mailname, 0);
 	if (i > 0 && value("emptystart") == NULL)
 		exit(1);
 	if (sigsetjmp(hdrjmp, 1) == 0) {
@@ -422,11 +467,12 @@ usage:
 		safe_signal(SIGINT, prevint);
 	}
 	commands();
-	if (mb.mb_type == MB_FILE) {
+	if (mb.mb_type == MB_FILE || mb.mb_type == MB_MAILDIR) {
 		safe_signal(SIGHUP, SIG_IGN);
 		safe_signal(SIGINT, SIG_IGN);
 		safe_signal(SIGQUIT, SIG_IGN);
 	}
+	strncpy(mboxname, expand("&"), sizeof mboxname)[sizeof mboxname-1]='\0';
 	quit();
 	return exit_status;
 }
@@ -435,9 +481,8 @@ usage:
  * Interrupt printing of the headers.
  */
 /*ARGSUSED*/
-static void
-hdrstop(signo)
-	int signo;
+static void 
+hdrstop(int signo)
 {
 
 	fflush(stdout);
@@ -453,8 +498,9 @@ hdrstop(signo)
  *	If baud rate > 1200, use 24 or ws_row
  * Width is either 80 or ws_col;
  */
-static void
-setscreensize()
+/*ARGSUSED*/
+static void 
+setscreensize(int dummy)
 {
 	struct termios tbuf;
 #ifdef	TIOCGWINSZ
@@ -463,7 +509,7 @@ setscreensize()
 	speed_t ospeed;
 
 #ifdef	TIOCGWINSZ
-	if (ioctl(1, TIOCGWINSZ, (char *) &ws) < 0)
+	if (ioctl(1, TIOCGWINSZ, &ws) < 0)
 		ws.ws_col = ws.ws_row = 0;
 #endif
 	if (tcgetattr(1, &tbuf) < 0)
