@@ -38,7 +38,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)send.c	2.78 (gritter) 12/26/04";
+static char sccsid[] = "@(#)send.c	2.81 (gritter) 3/4/05";
 #endif
 #endif /* not lint */
 
@@ -147,19 +147,23 @@ send(struct message *mp, FILE *obuf, struct ignoretab *doign,
 	count = mp->m_size;
 	sz = 0;
 	if (mp->m_flag & MNOFROM) {
-		if (doign != allignore && doign != fwdignore)
+		if (doign != allignore && doign != fwdignore &&
+				action != SEND_RFC822)
 			sz = fprintf(obuf, "%sFrom %s %s\n",
 					prefix ? prefix : "",
 					fakefrom(mp), fakedate(mp->m_time));
 	} else {
-		if (prefix && doign != allignore && doign != fwdignore) {
+		if (prefix && doign != allignore && doign != fwdignore &&
+				action != SEND_RFC822) {
 			fputs(prefix, obuf);
 			sz += strlen(prefix);
 		}
 		while (count && (c = getc(ibuf)) != EOF) {
-			if (doign != allignore && doign != fwdignore)
+			if (doign != allignore && doign != fwdignore &&
+					action != SEND_RFC822) {
 				putc(c, obuf);
-			sz++;
+				sz++;
+			}
 			count--;
 			if (c == '\n')
 				break;
@@ -168,7 +172,7 @@ send(struct message *mp, FILE *obuf, struct ignoretab *doign,
 	if (sz)
 		addstats(stats, 1, sz);
 	pf = 0;
-	if (action != SEND_MBOX && action != SEND_SHOW)
+	if (action != SEND_MBOX && action != SEND_RFC822 && action != SEND_SHOW)
 		pf |= PARSE_DECRYPT|PARSE_PARTS;
 	if ((ip = parsemsg(mp, pf)) == NULL)
 		return -1;
@@ -204,7 +208,8 @@ sendpart(struct message *zmp, struct mimepart *ip, FILE *obuf,
 	(void)&stats;
 	(void)&action;
 	if (ip->m_mimecontent == MIME_PKCS7 && ip->m_multipart &&
-			action != SEND_MBOX && action != SEND_SHOW)
+			action != SEND_MBOX && action != SEND_RFC822 &&
+			action != SEND_SHOW)
 		goto skip;
 	dostat = 0;
 	if (level == 0) {
@@ -388,9 +393,11 @@ skip:	switch (ip->m_mimecontent) {
 		case SEND_DECRYPT:
 			goto multi;
 		case SEND_TOFILE:
+		case SEND_TOPIPE:
 			put_from_(obuf, ip->m_multipart);
 			/*FALLTHRU*/
 		case SEND_MBOX:
+		case SEND_RFC822:
 		case SEND_SHOW:
 			break;
 		}
@@ -417,8 +424,8 @@ skip:	switch (ip->m_mimecontent) {
 			return rt;
 		break;
 	case MIME_PKCS7:
-		if (action != SEND_MBOX && action != SEND_SHOW &&
-				ip->m_multipart)
+		if (action != SEND_MBOX && action != SEND_RFC822 &&
+				action != SEND_SHOW && ip->m_multipart)
 			goto multi;
 		/*FALLTHRU*/
 	default:
@@ -441,9 +448,11 @@ skip:	switch (ip->m_mimecontent) {
 				break;
 			return -1;
 		case SEND_TOFILE:
+		case SEND_TOPIPE:
 		case SEND_TOSRCH:
 		case SEND_DECRYPT:
 		case SEND_MBOX:
+		case SEND_RFC822:
 		case SEND_SHOW:
 			break;
 		}
@@ -470,6 +479,7 @@ skip:	switch (ip->m_mimecontent) {
 		case SEND_QUOTE:
 		case SEND_QUOTE_ALL:
 		case SEND_TOFILE:
+		case SEND_TOPIPE:
 		case SEND_TOSRCH:
 		case SEND_TOFLTR:
 		case SEND_DECRYPT:
@@ -517,10 +527,12 @@ skip:	switch (ip->m_mimecontent) {
 					putc('\0', obuf);
 					/*FALLTHRU*/
 				case SEND_MBOX:
+				case SEND_RFC822:
 				case SEND_SHOW:
 				case SEND_TOSRCH:
 				case SEND_QUOTE:
 				case SEND_DECRYPT:
+				case SEND_TOPIPE:
 					break;
 				}
 				if (sendpart(zmp, np, obuf,
@@ -541,6 +553,7 @@ skip:	switch (ip->m_mimecontent) {
 			}
 			return rt;
 		case SEND_MBOX:
+		case SEND_RFC822:
 		case SEND_SHOW:
 			break;
 		}
@@ -577,13 +590,16 @@ skip:	switch (ip->m_mimecontent) {
 		convert = CONV_NONE;
 	}
 	if (action == SEND_DECRYPT || action == SEND_MBOX ||
-			action == SEND_SHOW)
+			action == SEND_RFC822 || action == SEND_SHOW)
 		convert = CONV_NONE;
 	tcs = gettcharset();
 #ifdef	HAVE_ICONV
 	if ((action == SEND_TODISP || action == SEND_TODISP_ALL ||
 			action == SEND_QUOTE || action == SEND_QUOTE_ALL ||
-			action == SEND_TOSRCH) && pipecmd == NULL) {
+			action == SEND_TOSRCH) &&
+			(ip->m_mimecontent == MIME_TEXT_PLAIN ||
+			 ip->m_mimecontent == MIME_TEXT_HTML ||
+			 ip->m_mimecontent == MIME_TEXT)) {
 		if (iconvd != (iconv_t)-1)
 			iconv_close(iconvd);
 		if (asccasecmp(tcs, ip->m_charset) &&
@@ -596,10 +612,10 @@ skip:	switch (ip->m_mimecontent) {
 	if ((action == SEND_TODISP || action == SEND_TODISP_ALL ||
 			action == SEND_QUOTE || action == SEND_QUOTE_ALL) &&
 			pipecmd != NULL) {
-		action = SEND_TOFILE;
 		qbuf = obuf;
 		pbuf = getpipefile(pipecmd, &qbuf,
 			action == SEND_QUOTE || action == SEND_QUOTE_ALL);
+		action = SEND_TOPIPE;
 		if (pbuf != qbuf) {
 			oldpipe = safe_signal(SIGPIPE, onpipe);
 			if (sigsetjmp(pipejmp, 1))
@@ -624,7 +640,8 @@ skip:	switch (ip->m_mimecontent) {
 			free(line2);
 		}
 		out(line, linelen, pbuf, convert, action,
-				prefix, prefixlen,
+				pbuf == origobuf ? prefix : NULL,
+				pbuf == origobuf ? prefixlen : 0,
 				pbuf == origobuf ? stats : NULL);
 		if (ferror(pbuf)) {
 			rt = -1;
@@ -921,7 +938,7 @@ out(char *buf, size_t len, FILE *fp,
 					action == SEND_QUOTE ||
 					action == SEND_QUOTE_ALL ?
 				TD_ISPR|TD_ICONV :
-				action == SEND_TOSRCH ?
+				action == SEND_TOSRCH || action == SEND_TOPIPE ?
 					TD_ICONV :
 				action == SEND_TOFLTR ?
 					TD_DELCTRL :
