@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * Copyright (c) 1980, 1993
@@ -38,7 +38,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)strings.c	2.1 (gritter) 9/1/02";
+static char sccsid[] = "@(#)strings.c	2.5 (gritter) 10/2/04";
 #endif
 #endif /* not lint */
 
@@ -61,9 +61,8 @@ static char sccsid[] = "@(#)strings.c	2.1 (gritter) 9/1/02";
  * the occasional user with enormous string size requests.
  */
 
-char *
-salloc(size)
-	int size;
+void *
+salloc(size_t size)
 {
 	char *t;
 	int s;
@@ -95,13 +94,23 @@ salloc(size)
 	return(t);
 }
 
+void *
+csalloc(size_t nmemb, size_t size)
+{
+	void	*vp;
+
+	vp = salloc(nmemb * size);
+	memset(vp, 0, nmemb * size);
+	return vp;
+}
+
 /*
  * Reset the string area to be empty.
  * Called to free all strings allocated
  * since last reset.
  */
-void
-sreset()
+void 
+sreset(void)
 {
 	struct strings *sp;
 	int string_index;
@@ -122,8 +131,8 @@ sreset()
  * Make the string area permanent.
  * Meant to be called in main, after initialization.
  */
-void
-spreserve()
+void 
+spreserve(void)
 {
 	struct strings *sp;
 
