@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * Copyright (c) 1980, 1993
@@ -38,7 +38,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)vars.c	2.1 (gritter) 9/1/02";
+static char sccsid[] = "@(#)vars.c	2.10 (gritter) 10/2/04";
 #endif
 #endif /* not lint */
 
@@ -51,23 +51,45 @@ static char sccsid[] = "@(#)vars.c	2.1 (gritter) 9/1/02";
  * Variable handling stuff.
  */
 
-static void	vfree __P((char *));
-static struct var	*lookup __P((char []));
-static void	remove_grouplist __P((struct grouphead *));
+static char *canonify(const char *vn);
+static void vfree(char *cp);
+static struct var *lookup(const char *name);
+static void remove_grouplist(struct grouphead *gh);
+
+/*
+ * If a variable name begins with a lowercase-character and contains at
+ * least one '@', it is converted to all-lowercase. This is necessary
+ * for lookups of names based on email addresses.
+ *
+ * Following the standard, only the part following the last '@' should
+ * be lower-cased, but practice has established otherwise here.
+ */
+static char *
+canonify(const char *vn)
+{
+	const char	*vp;
+
+	if (upperchar(*vn&0377))
+		return (char *)vn;
+	for (vp = vn; *vp && *vp != '@'; vp++);
+	if (*vp == '@')
+		return i_strdup(vn);
+	return (char *)vn;
+}
 
 /*
  * Assign a value to a variable.
  */
-void
-assign(name, value)
-	char name[], value[];
+void 
+assign(char *name, char *value)
 {
 	struct var *vp;
 	int h;
 
+	name = canonify(name);
 	h = hash(name);
 	vp = lookup(name);
-	if (vp == NOVAR) {
+	if (vp == NULL) {
 		vp = (struct var *)scalloc(1, sizeof *vp);
 		vp->v_name = vcopy(name);
 		vp->v_link = variables[h];
@@ -83,9 +105,8 @@ assign(name, value)
  * strings whose value is "" since they are expected to be frequent.
  * Thus, we cannot free same!
  */
-static void
-vfree(cp)
-	char *cp;
+static void 
+vfree(char *cp)
 {
 	if (*cp)
 		free(cp);
@@ -97,8 +118,7 @@ vfree(cp)
  */
 
 char *
-vcopy(str)
-	char str[];
+vcopy(const char *str)
 {
 	char *new;
 	unsigned len;
@@ -106,7 +126,7 @@ vcopy(str)
 	if (*str == '\0')
 		return "";
 	len = strlen(str) + 1;
-	new = (char*)smalloc(len);
+	new = smalloc(len);
 	memcpy(new, str, (int) len);
 	return new;
 }
@@ -117,14 +137,18 @@ vcopy(str)
  */
 
 char *
-value(name)
-	char name[];
+value(const char *name)
 {
 	struct var *vp;
+	char	*vs;
 
-	if ((vp = lookup(name)) == NOVAR)
-		return(getenv(name));
-	return(vp->v_value);
+	name = canonify(name);
+	if ((vp = lookup(name)) == NULL) {
+		if ((vs = getenv(name)) != NULL && *vs)
+			vs = savestr(vs);
+		return vs;
+	}
+	return vp->v_value;
 }
 
 /*
@@ -133,15 +157,14 @@ value(name)
  */
 
 static struct var *
-lookup(name)
-	char name[];
+lookup(const char *name)
 {
 	struct var *vp;
 
-	for (vp = variables[hash(name)]; vp != NOVAR; vp = vp->v_link)
+	for (vp = variables[hash(name)]; vp != NULL; vp = vp->v_link)
 		if (*vp->v_name == *name && equal(vp->v_name, name))
 			return(vp);
-	return(NOVAR);
+	return(NULL);
 }
 
 /*
@@ -149,45 +172,42 @@ lookup(name)
  */
 
 struct grouphead *
-findgroup(name)
-	char name[];
+findgroup(char *name)
 {
 	struct grouphead *gh;
 
-	for (gh = groups[hash(name)]; gh != NOGRP; gh = gh->g_link)
+	for (gh = groups[hash(name)]; gh != NULL; gh = gh->g_link)
 		if (*gh->g_name == *name && equal(gh->g_name, name))
 			return(gh);
-	return(NOGRP);
+	return(NULL);
 }
 
 /*
  * Print a group out on stdout
  */
-void
-printgroup(name)
-	char name[];
+void 
+printgroup(char *name)
 {
 	struct grouphead *gh;
 	struct group *gp;
 
-	if ((gh = findgroup(name)) == NOGRP) {
+	if ((gh = findgroup(name)) == NULL) {
 		printf(catgets(catd, CATSET, 202, "\"%s\": not a group\n"),
 				name);
 		return;
 	}
 	printf("%s\t", gh->g_name);
-	for (gp = gh->g_list; gp != NOGE; gp = gp->ge_link)
+	for (gp = gh->g_list; gp != NULL; gp = gp->ge_link)
 		printf(" %s", gp->ge_name);
-	sputc('\n', stdout);
+	putchar('\n');
 }
 
 /*
  * Hash the passed string and return an index into
  * the variable or group hash table.
  */
-int
-hash(name)
-	const char *name;
+int 
+hash(const char *name)
 {
 	int h = 0;
 
@@ -200,15 +220,15 @@ hash(name)
 	return (h % HSHSIZE);
 }
 
-int
-unset_internal(name)
-	char *name;
+int 
+unset_internal(char *name)
 {
 	struct var *vp, *vp2;
 	int h;
 
-	if ((vp2 = lookup(name)) == NOVAR) {
-		if (!sourcing) {
+	name = canonify(name);
+	if ((vp2 = lookup(name)) == NULL) {
+		if (!sourcing && !unset_allow_undefined) {
 			printf(catgets(catd, CATSET, 203,
 				"\"%s\": undefined variable\n"), name);
 			return 1;
@@ -220,14 +240,14 @@ unset_internal(name)
 		variables[h] = variables[h]->v_link;
 		vfree(vp2->v_name);
 		vfree(vp2->v_value);
-		free((char *)vp2);
+		free(vp2);
 		return 0;
 	}
 	for (vp = variables[h]; vp->v_link != vp2; vp = vp->v_link);
 	vp->v_link = vp2->v_link;
 	vfree(vp2->v_name);
 	vfree(vp2->v_value);
-	free((char *) vp2);
+	free(vp2);
 	return 0;
 }
 
@@ -251,7 +271,7 @@ remove_group(const char *name)
 	struct grouphead *gh, *gp = NULL;
 	int h = hash(name);
 
-	for (gh = groups[h]; gh != NOGRP; gh = gh->g_link) {
+	for (gh = groups[h]; gh != NULL; gh = gh->g_link) {
 		if (*gh->g_name == *name && equal(gh->g_name, name)) {
 			remove_grouplist(gh);
 			vfree(gh->g_name);
